-- 01-05-2003 10-08-2011 20-04-2019 pmh
-- database funktion - ms_sql version - v11_3 - genindlagt
-- Anlg - Tilladelser - Indvindinger

patch_id rp_wcat_plant_perm_v11_3

--******************************************************************************
--  rp_wcat_plant_perm_date
--******************************************************************************

create function geokon.rp_wcat_plant_perm_date (
   @site_seq_no integer,
   @mode varchar(6),
   @perm_date datetime)
   RETURNS numeric(9,0)
-- returnerer indvindingstilladelse pr. anlg - numeric(9)
-- param 1:  site_seq_no - integer
-- param 2:  mode - '', 'plant', 'area'
-- param 3:  pr. dato - datetime
BEGIN
   DECLARE @amount_1 numeric(9,0);
   DECLARE @amount_2 numeric(9,0);

   if (@mode <> 'area') BEGIN
      SET @amount_1 = (
         SELECT cp.amount_year
           FROM wcat_catch_permission cp
          WHERE cp.site_seq_no = @site_seq_no
            AND cp.gw_catchment_area = '0000'
            AND coalesce(cp.revoked,'0') = '0'
            AND (cp.expire_date is null or cp.expire_date >= @perm_date)
            AND cp.perm_date = (SELECT max(c2.perm_date)
                                  FROM wcat_catch_permission c2
                                 WHERE c2.site_seq_no = @site_seq_no
                                   AND c2.gw_catchment_area = '0000'
                                   AND coalesce(c2.revoked,'0') = '0'
                                   AND c2.perm_date <= @perm_date)
      );
      IF (@mode = 'plant' OR @amount_1 >= 0) RETURN (@amount_1);
   END;
 
   SET @amount_1 = (
      SELECT sum(cp.amount_year)
        FROM wcat_catch_permission cp
       WHERE cp.site_seq_no = @site_seq_no
         AND cp.gw_catchment_area <> '0000'
         AND coalesce(cp.revoked,'0') = '0'
         AND (cp.expire_date is null or cp.expire_date >= @perm_date)
         AND cp.perm_date = (SELECT max(c2.perm_date)
                               FROM wcat_catch_permission c2
                              WHERE c2.site_seq_no = @site_seq_no
                                AND c2.gw_catchment_area = cp.gw_catchment_area
                                AND coalesce(c2.revoked,'0') = '0'
                                AND c2.perm_date <= @perm_date)
   );
   SET @amount_2 = (
      SELECT sum(bp.amount_year)
        FROM bor_borings bb
        JOIN wcat_bor_permission bp
             ON (bp.boring_seq_no = bb.boring_seq_no)
        LEFT OUTER JOIN wcat_bor_catchment bc
             ON (bc.boring_seq_no = bb.boring_seq_no) 
       WHERE bb.site_seq_no = @site_seq_no
         AND (bb.enddate is null or bb.enddate >= @perm_date)
         AND coalesce(bp.revoked,'0') = '0'
         AND (bp.expire_date is null or bp.expire_date >= @perm_date)
         AND bp.perm_date = (SELECT max(b2.perm_date)
                               FROM wcat_bor_permission b2
                              WHERE b2.boring_seq_no = bb.boring_seq_no
                                AND coalesce(b2.revoked,'0') = '0'
                                AND b2.perm_date <= @perm_date)
         AND geokon.rp_wcat_area_perm_date (bb.site_seq_no, bc.gw_catchment_area, @perm_date) is null
    );
    IF (@amount_1 is null) RETURN (@amount_2);
    IF (@amount_2 is null) RETURN (@amount_1);
    RETURN (@amount_1 + @amount_2); 
END;

--******************************************************************************
--  rp_wcat_plant_perm_year
--******************************************************************************

create function geokon.rp_wcat_plant_perm_year (
   @site_seq_no integer,
   @perm_year char(4))
   RETURNS numeric(9,0)
-- returnerer indvindingstilladelse pr. anlg - numeric(9)
-- param 1:  site_seq_no - integer
-- param 2:  indberetningsr - char(4)
BEGIN
   DECLARE @amount_1 numeric(9,0);
   DECLARE @amount_2 numeric(9,0);

   SET @amount_1 = (
      SELECT cp.amount_year
        FROM wcat_catch_permission cp
       WHERE cp.site_seq_no = @site_seq_no
         AND cp.gw_catchment_area = '0000'
         AND coalesce(cp.revoked,'0') = '0'
         AND (cp.expire_date is null or year(cp.expire_date) >= convert(int, @perm_year))
         AND cp.perm_date = (SELECT max(c2.perm_date)
                               FROM wcat_catch_permission c2
                              WHERE c2.site_seq_no = @site_seq_no
                                AND c2.gw_catchment_area = '0000'
                                AND coalesce(c2.revoked,'0') = '0'
                                AND year(c2.perm_date) <= convert(int, @perm_year))
   );
   IF (@amount_1 >= 0) RETURN (@amount_1)
  
   SET @amount_1 = (
      SELECT sum(cp.amount_year)
        FROM wcat_catch_permission cp
       WHERE cp.site_seq_no = @site_seq_no
         AND cp.gw_catchment_area <> '0000'
         AND coalesce(cp.revoked,'0') = '0'
         AND (cp.expire_date is null or year(cp.expire_date) >= convert(int, @perm_year))
         AND cp.perm_date = (SELECT max(c2.perm_date)
                               FROM wcat_catch_permission c2
                              WHERE c2.site_seq_no = @site_seq_no
                                AND c2.gw_catchment_area = cp.gw_catchment_area
                                AND coalesce(c2.revoked,'0') = '0'
                                AND year(c2.perm_date) <= convert(int, @perm_year))
   );
   SET @amount_2 = (
      SELECT sum(bp.amount_year)
        FROM bor_borings bb
        JOIN wcat_bor_permission bp
          ON (bp.boring_seq_no = bb.boring_seq_no)
        LEFT OUTER JOIN wcat_bor_catchment bc
          ON (bc.boring_seq_no = bb.boring_seq_no) 
       WHERE bb.site_seq_no = @site_seq_no
         AND (bb.enddate is null or year(bb.enddate) >= convert(int, @perm_year))
         AND coalesce(bp.revoked,'0') = '0'
         AND (bp.expire_date is null or year(bp.expire_date) >= convert(int, @perm_year))
         AND bp.perm_date = (SELECT max(b2.perm_date)
                               FROM wcat_bor_permission b2
                              WHERE b2.boring_seq_no = bb.boring_seq_no
                                AND coalesce(b2.revoked,'0') = '0'
                                AND year(b2.perm_date) <= convert(int, @perm_year))
         AND geokon.rp_wcat_area_perm_year (bb.site_seq_no, bc.gw_catchment_area, @perm_year) is null
   );
    IF (@amount_1 is null) RETURN (@amount_2);
   IF (@amount_2 is null) RETURN (@amount_1);
   RETURN (@amount_1 + @amount_2); 
END;

--******************************************************************************
grant execute on geokon.rp_wcat_plant_perm_date to public;
grant execute on geokon.rp_wcat_plant_perm_year to public;
--******************************************************************************
